<?php

/*****************************************************************************/
function getint1 ($a, &$p)
/*****************************************************************************/
{
	/* Description: Read a 1-byte unsigned integer.
	 * Parameters:
	 * - $a: Source binary string.
	 * - $p: Offset.
	 * Returns: integer
	 */

	$return = ord ($a[$p++]);

	return ($return);
}
/*****************************************************************************/
function getch ($a, &$p)
/*****************************************************************************/
{
	/* Description: Read a character.
	 * Parameters:
	 * - $a: Source binary string.
	 * - $p: Offset.
	 * Returns: character
	 */

	$return = $a[$p++];

	return ($return);
}
/*****************************************************************************/
function getint2 ($a, &$p)
/*****************************************************************************/
{
	/* Description: Read a 2-byte unsigned integer. (little-endian)
	 * Parameters:
	 * - $a: Source binary string.
	 * - $p: Offset.
	 * Returns: integer
	 */

	$return = ord ($a[$p++]) + (ord ($a[$p++]) << 8);

	return ($return);
}
/*****************************************************************************/
function getint2mac ($a, &$p)
/*****************************************************************************/
{
	/* Description: Read a 2-byte unsigned integer. (big-endian)
	 * Parameters:
	 * - $a: Source binary string.
	 * - $p: Offset.
	 * Returns: integer
	 */

	$return = (ord ($a[$p++]) << 8) + ord ($a[$p++]);

	return ($return);
}
/*****************************************************************************/
function int42str ($a)
/*****************************************************************************/
{
	/* Description: Convert a 4-byte integer to binary. (little-endian)
	 * Parameters:
	 * - $a: The integer.
	 * Returns: string
	 */

	$return = '';

	$return .= chr ($a & 255);
	$return .= chr (($a >> 8) & 255);
	$return .= chr (($a >> 16) & 255);
	$return .= chr (($a >> 24) & 255);

	return ($return);
}
/*****************************************************************************/
function int22str ($a)
/*****************************************************************************/
{
	/* Description: Convert a 2-byte integer to binary. (little-endian)
	 * Parameters:
	 * - $a: The integer.
	 * Returns: string
	 */

	$return = '';

	$return .= chr ($a & 255);
	$return .= chr (($a >> 8) & 255);

	return ($return);
}
/*****************************************************************************/
function getstr ($a, &$p, $len)
/*****************************************************************************/
{
	/* Description: Read a string with known length.
	 * Parameters:
	 * - $a: Source binary string.
	 * - $p: Offset.
	 * - $len: Length.
	 * Returns: The string.
	 */

	$s = substr ($a, $p, $len);
	$p += $len;

	return ($s);
}

/*****************************************************************************/
function putstr (&$f, &$p, $s)
/*****************************************************************************/
{
	/* Description: Overwrite part of a string.
	 * Parameters:
	 * - $f: Destination binary string.
	 * - $p: Offset.
	 * - $s: The source string that is inserted.
	 * Returns: Nothing.
	 */

	for ($i = 0; $i < strlen ($s); $i++)
		{ $f[$p++] = $s[$i]; }
}

?>