<?php

require_once("binary.php");

/*****************************************************************************/
function write_bmp ($fn, $width, $height, $data, $pal, $bpp, $data_stride)
/*****************************************************************************/
{
	/* Description: Create a BMP file.
	 * Parameters:
	 * - $fn: File name.
	 * - $width: Width in pixels.
	 * - $height: Height in pixels.
	 * - $data: Pixel data.
	 * - $pal: Palette. (if $bpp<=8)
	 * - $bpp: Bits per pixel. (must be one of: 1,4,8,16,24,32)
	 * - $data_stride: Width of pixel data ($data) in bytes. (also called stride or pitch)
	 * Returns: Nothing.
	 */

	$bmp_stride = (($width * $bpp + 31)&~31) >> 3; // stride in BMP (round up to multiple of 32 bits)
	$row_size = (($width * $bpp + 7)&~7) >> 3; // row size in BMP (round up to multiple of 8 bits)
	$palette_length = ($bpp <= 8) ? (4 << $bpp) : 0; // palette length

	// BITMAPFILEHEADER
	$bmp = "BM"; // bfType
	$bmp .= int42str (0x36 + $bmp_stride * $height + $palette_length); // bfSize
	$bmp .= int22str (0); // bfReserved1
	$bmp .= int22str (0); // bfReserved2
	$bmp .= int42str (0x36 + $palette_length); // bfOffBits
	// BITMAPINFOHEADER
	$bmp .= int42str (0x28); // biSize
	$bmp .= int42str ($width); // biWidth
	$bmp .= int42str ($height); // biHeight
	$bmp .= int22str (1); // biPlanes
	$bmp .= int22str ($bpp); // biBitCount
	$bmp .= int42str (0); // biCompression
	$bmp .= int42str ($bmp_stride * $height); // biSizeImage
	$bmp .= int42str (0xEC4); // biXPelsPerMeter
	$bmp .= int42str (0xEC4); // biYPelsPerMeter
	$bmp .= int42str (0); // biClrUsed
	$bmp .= int42str (0); // biClrImportant
	// palette
	$pal = substr ($pal, 0, $palette_length); // if it was longer
	$pal = str_pad ($pal, $palette_length, "\x00"); // if it was shorter
	$bmp .= $pal;

	$pad = str_repeat ("\x00", $bmp_stride - $row_size); // padding in BMP

	// Scanlines. Bottom to top, as BMP expects it.
	for ($y = 0; $y < $height; $y++)
	{
		$bmp .= substr ($data, $data_stride * ($height - $y - 1), $row_size);
		$bmp .= $pad;
	}

	$fp_out = fopen ($fn, "wb");
	fwrite ($fp_out, $bmp);
	fclose ($fp_out);
}

?>