<?php

require_once("binary.php");

/*****************************************************************************/
function decode_level_part ($f, &$p, $ln)
/*****************************************************************************/
{
	/* Description: Decode a level.
	 * Parameters:
	 * - $f: ROM image.
	 * - $p: Offset.
	 * - $ln: Length of part.
	 * Returns: The decoded level.
	 */

	$p0 = $p;
	$out = "";
	$end = false;
	while (strlen ($out) < $ln and ! $end){
		$x = getint1 ($f, $p);
		if ($x & 0x80) {
			$out .= getstr($f, $p, $x & 0x7F);
		} else {
			$n = getint1($f,$p);
			if (!$n) $n = 256;
			switch ($x) {
				case 1: $c=getch($f,$p); $out.=str_repeat($c,$n); if($n==0) $end=true; break;
				case 2: $c=getint1($f,$p); for($i=0;$i<$n;$i++) $out.=chr($c++); break;
				case 3: $c=getint1($f,$p); for($i=0;$i<$n;$i++) $out.=chr($c--); break;
				case 4: $c=getint1($f,$p); for($i=0;$i<$n;$i++) $out.=$out[$c++]; break;
				case 5: $c=getint2($f,$p); for($i=0;$i<$n;$i++) $out.=$out[$c++]; break;
				default:
					print "Unknown level compression: " . $x . " at offset 0x" . dechex ($p) . "\n";
					$end=true;
			}
		}
	}
	return $out;
}

/*****************************************************************************/
function decode_level ($f, &$p)
/*****************************************************************************/
{
	/* Description: Decode a level.
	 * Parameters:
	 * - $f: ROM image.
	 * - $p: Offset.
	 * Returns: The decoded level.
	 */

	$out = "";
	$out .= decode_level_part($f, $p, 720); // background layer (layer 1)
	$out .= decode_level_part($f, $p, 720); // foreground blocks layer (layers 2 and 3)
	$out .= decode_level_part($f, $p, 720); // modifiers layer
	$out .= decode_level_part($f, $p, 256); // block layer 2 graphics
	$out .= decode_level_part($f, $p, 256); // block layer 3 graphics
	$out .= decode_level_part($f, $p, 256); // block objects
	$out .= decode_level_part($f, $p, 256); // block flags and background animation info
	$out .= decode_level_part($f, $p, 244); // room links, starting position, guards
	$out .= decode_level_part($f, $p, 256); // door links 1
	$out .= decode_level_part($f, $p, 256); // door links 2
	return $out;
}

?>