<?php

require_once("binary.php");

/*****************************************************************************/
function decode_screen_tilemap ($f, &$p, $si)
/*****************************************************************************/
{
	/* Description: Decode tilemap for full-screen image.
	 * Parameters:
	 * - $f: ROM image.
	 * - $p: Offset.
	 * - $si: Size of expected result in bytes.
	 * Returns: The tilemap in SNES format.
	 */

	$out = str_repeat ("\x00", $si);
	$o = 0;
	$end = false;

	while (($o < $si) and (!$end))
	{
		$t = getint2 ($f, $p);
		if ($t&0x2000)
		{
			switch ($t&0xE000)
			{
				case 0x2000: // repeat
					$t &= 0x1FFF;
					$ti = getint2 ($f, $p);
					$something = str_repeat (int22str ($ti), $t);
					putstr ($out, $o, $something);
					break;
				case 0x6000: // incrementing sequence
					$t &= 0x1FFF;
					$ti = getint2 ($f, $p);
					for ($i = 0; $i < $t; $i++)
					{
						$something = int22str ($ti++);
						putstr ($out, $o, $something);
					}
					break;
				case 0xA000: // decrementing sequence
					$t &= 0x1FFF;
					$ti = getint2 ($f, $p);
					for ($i = 0; $i < $t; $i++)
					{
						$something = int22str ($ti--);
						putstr ($out, $o, $something);
					}
					break;
				default:
					printf ("decode_screen_tilemap: 0x%04X at offset %X\n", $t, $p);
			}
		} else {
			$something = int22str ($t);
			putstr ($out, $o, $something);
		}
	}

	return ($out);
}

/*****************************************************************************/
function decode_level_blocks_wide ($f, &$p, $si)
/*****************************************************************************/
{
	/* Description: Decode blocks for level graphics. (layer 2)
	 * Parameters:
	 * - $f: ROM image.
	 * - $p: Offset.
	 * - $si: Size of expected result in bytes.
	 * Returns: The blocks in SNES tilemap format.
	 */

	$out = str_repeat ("\x00", $si);
	$o = 0;
	$end = false;

	while (($o < $si) and (!$end))
	{
		$t = getint1 ($f, $p);
		if ($t&0x20)
		{
			switch ($t&0xE0)
			{
				case 0x20: // repeat
					$p--;
					$t = getint2mac ($f, $p);
					$t &= 0x1FFF;
					$ti = getint2mac ($f, $p);
					$something = str_repeat (int22str ($ti), $t);
					putstr ($out, $o, $something);
					break;
				case 0x60: // incrementing sequence
					$p--;
					$t = getint2mac ($f, $p);
					$t &= 0x1FFF;
					$ti = getint2mac ($f, $p);
					for ($i = 0; $i < $t; $i++)
					{
						$something = int22str ($ti++);
						putstr ($out, $o, $something);
					}
					break;
				case 0xA0: // copy earlier
					$t &= 0x1F;
					$something = substr ($out, $o - $o%0x60 + $t * 2, 2);
					putstr ($out, $o, $something);
					break;
				case 0xE0: // copy earlier
					$t &= 0x1F;
					$something = substr ($out, $o - $o%0x60 + ($t + 0x20) * 2, 2);
					putstr ($out, $o, $something);
					break;
				default:
					printf ("decode_level_blocks_wide: 0x%02X at offset 0x%X\n", $t, $p);
			}
		} else {
			$p--;
			$something = int22str (getint2mac ($f, $p));
			putstr ($out, $o, $something);
		}
	}

	return ($out);
}

/*****************************************************************************/
function decode_level_blocks_narrow ($f, &$p, $si)
/*****************************************************************************/
{
	/* Description: Decode blocks for level graphics. (layers 1 and 3)
	 * Parameters:
	 * - $f: ROM image.
	 * - $p: Offset.
	 * - $si: Size of expected result in bytes.
	 * Returns: The blocks in SNES tilemap format.
	 */

	$out = str_repeat ("\x00", $si);
	$o = 0;
	$end = false;

	while (($o < $si) and (!$end))
	{
		$t = getint1 ($f, $p);
		if ($t & 0x20)
		{
			switch ($t & 0xE0)
			{
				case 0x20: // repeat
					$t &= 0x1F;
					$ti = getint2mac ($f, $p);
					$something = str_repeat (int22str ($ti), $t);
					putstr ($out, $o, $something);
					break;
				case 0x60: // incrementing sequence
					$t &= 0x1F;
					$ti = getint2mac ($f, $p);
					for ($i = 0; $i < $t; $i++)
					{
						$something = int22str ($ti++);
						putstr ($out, $o, $something);
					}
					break;
				case 0xA0: // copy earlier
					$t &= 0x1F;
					$something = substr ($out, $o - $o%0x30 + $t * 2, 2);
					putstr ($out, $o, $something);
					break;
				default:
					printf ("decode_level_blocks_narrow: 0x%02X at offset 0x%X\n", $t, $p);
			}
		} else {
			$p--;
			$something = int22str (getint2mac ($f, $p));
			putstr ($out, $o, $something);
		}
	}

	return ($out);
}

?>