<?php

require_once("binary.php");

/*****************************************************************************/
function make_map ($map, $tiles, $map_width, $map_height, $tile_width, $tile_height, $first_tile, $item_size, &$image_width_out, &$image_height_out)
/*****************************************************************************/
{
	/* Description: Assembles smaller images (the tiles) into a bigger image, according to the tilemap.
	 * Parameters:
	 * IN:
	 * - $map: The tilemap.
	 * - $tiles: The tiles.
	 * - $map_width, $map_height: Map width, height (in tiles).
	 * - $tile_width, $tile_height: Tile width, height (in bytes).
	 * - $first_tile: Index of first tile (usually 0 or 1).
	 * - $item_size: Size of a map item in bytes (usually 1 or 2).
	 * OUT:
	 * - $image_width, $image_height: Size of the final image.
	 * Returns: The resulting image.
	 */

	$image_width = $map_width * $tile_width; // image width in bytes
	$image_height = $map_height * $tile_height; // image height
	$image_size = $image_width * $image_height; // image size in bytes
	$tile_size = $tile_width * $tile_height; // tile size in bytes
	$image_width_out = $image_width;
	$image_height_out = $image_height;

	$out = str_repeat("\x00",$image_size);

	// jumps in the output:
	$e1 = $image_width - $tile_width; // end of pixel line in a tile
	$e2 = $tile_width - $tile_height * $image_width; // end of tile
	$e3 = $image_width * ($tile_height - 1); // end of a line of tiles

	$p = $o = 0; // input/output position
	$get_item = "getint" . $item_size; // name of a function from binary.php
	$n_tiles = (integer)(strlen($tiles) / $tile_size); // number of tiles

	for ($tile_y = 0; $tile_y < $map_height; $tile_y++ ) // for each tile row
	{
		for ($tile_x = 0; $tile_x < $map_width; $tile_x++) // for each tile
		{
			$tile_index = $get_item($map, $p) - $first_tile;
			if ($tile_index >= 0 and $tile_index < $n_tiles)
			{
				for ($pixel_y = 0; $pixel_y < $tile_height; $pixel_y++) // for each pixel row
				{
					putstr ($out, $o, substr ($tiles, $tile_index * $tile_size + $pixel_y * $tile_width, $tile_width));
					$o += $e1;
				}
				$o += $e2;
			} else $o += $tile_width;
		}
		$o += $e3;
	}
	return $out;
}

?>