<?php

require_once("binary.php");

/*****************************************************************************/
function decode_mapping ($f, &$p, $value_bits, $n)
/*****************************************************************************/
{
	/* Description: Decodes an item-mapping (used in level graphics).
	 * Parameters:
	 * - $f: ROM image.
	 * - $p: Offset.
	 * - $value_bits: The size of the value part in an item.
	 * - $n: Expected length. Usually $n = 1 << $value_bits .
	 * Returns: The decoded item-mapping.
	 *          The purpose of this thing is to store duplicated tiles and blocks only once.
	 *          It's an array of indexes, telling which actual item is loaded from which stored item.
	 */

	/*
	 * Structure of an item:
	 * 1 [..length] [value] --> incrementing sequence
	 * 0 1 [length] [value] --> repeated value
	 * 0 0 ........ [value] --> single value
	*/

	$value_max = 1 << $value_bits;
	$value_mask = $value_max - 1;
	$length_increase_bits = 15 - $value_bits;
	$length_increase_max = 1 << $length_increase_bits;
	$length_increase_mask = $length_increase_max - 1;
	$length_repeat_bits = 14 - $value_bits;
	$length_repeat_max = 1 << $length_repeat_bits;
	$length_repeat_mask = $length_repeat_max - 1;

	$mapping = array();

	while (count ($mapping) < $n)
	{
		$x = getint2 ($f, $p);
		$value = $x & $value_mask;
		if ($x & 0x8000)
		{ // incrementing sequence
			$length = ($x >> $value_bits) & $length_increase_mask;
			if (!$length) { $length = $length_increase_max; }
			for ($i = 0; $i < $length; $i++) { $mapping[] = $value++; }
		} else {
			if ($x & 0x4000)
			{ // repeated value
				$length = ($x >> $value_bits) & $length_repeat_mask;
				if (!$length) { $length = $length_repeat_max; }
				for ($i = 0; $i < $length; $i++) { $mapping[] = $value; }
			} else { // single value
				$mapping[] = $value;
			}
		}
	}

	return $mapping;
}

/*****************************************************************************/
function apply_mapping ($mapping, $stored, $item_size)
/*****************************************************************************/
{
	/* Description: Apply item-mapping to some items.
	 * Parameters:
	 * - $mapping: The item-mapping.
	 * - $stored: The stored items.
	 * - $item_size: Size of an item.
	 * Returns: The actual items.
	 */
	$return = "";
	for ($i = 0; $i < count ($mapping); $i++)
	{
		$return .= substr ($stored, $mapping[$i] * $item_size, $item_size);
	}
	return $return;
}

?>