<?php

/*****************************************************************************/
function read_palette ($f, &$p, $length)
/*****************************************************************************/
{
	/* Description: Read a SNES palette and convert it to the format used in BMP files.
	 * Parameters:
	 * - $f: ROM image.
	 * - $p: Offset.
	 * - $length: Number of colors.
	 * Returns: The palette in BMP format.
	 */

	$pal = "";
	for ($c = 0; $c < $length; $c++)
	{
		$color = getint2 ($f, $p);
		$r = $color & 0x1F;
		$g = ($color >> 5) & 0x1F;
		$b = ($color >> 10) & 0x1F;
		$pal .= int42str ((($r << 16) | ($g << 8) | $b) << 3);
	}
	return $pal;
}

/*****************************************************************************/
function get_palette ($pal_res, $pal_index, $length)
/*****************************************************************************/
{
	/* Description: Read a SNES palette and convert it to the format used in BMP files.
	 * Parameters:
	 * - $pal_res: ID of palette resource or -1.
	 * - $pal_index: Index of palette within resource or -1. (This is an offset counted in 16 colors.)
	 * - $length: Number of colors.
	 * Returns: The palette in BMP format.
	 */

	if ($pal_res >= 0 and $pal_index >= 0)
	{
		// Convert a SNES palette resource.
		global $ad, $f;
		$p = $ad[0][$pal_res] + ($pal_index << 5);
		return read_palette ($f, $p, $length);
	}
	else
	{
		// Use default greyscale palette.
		$pal0 = "";
		for ($i = 0; $i < $length; $i++)
			{ $pal0 .= str_repeat (chr ($i * 255 / ($length - 1)), 3) . "\x00"; }
		return $pal0;
	}
}

?>