<?php

/* pop1s_export.php v0.2
 * Exports resources from the Prince of Persia for SNES ROM file.
 * Copyright (C) 2011-2016 Dávid Nagy (contact via forum.princed.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

require_once("binary.php");
require_once("resources.php");

/*****************************************************************************/
function export ($ix, $id)
/*****************************************************************************/
{
	/* Description: Export one resource.
	 * Parameters:
	 * - $ix: Index of resource table.
	 * - $id: Index of resource within table.
	 * Returns: Nothing.
	 */
	global $f, $ad;

	$p = $ad[$ix][$id];
	switch ($ix)
	{
		case 0: /*** Resource table 1. ***/
			if ($id == 0)
			{
				/*** 00: menu/intro font tiles (256 uncompressed 2bpp tiles) ***/
				export_tiles ($f, $p, $id);
			} else if (($id >= 1) and ($id <= 40)) {
				/*** 01..40: the 10 environments (graphics for level types) ***/
				switch ($id & 3)
				{
					/* Each type has four resources: background, blocks background,
					 * blocks foreground, palette.
					 */
					case 1: /*** background ***/
						export_blocks ($f, $p, $id, 2, 3, 8, 0x100, 0x200); break;
					case 2: /*** blocks background ***/
						export_blocks ($f, $p, $id, 4, 6, 8, 0x100, 0x100); break;
					case 3: /*** blocks foreground ***/
						export_blocks ($f, $p, $id, 4, 3, 8, 0x100, 0x100); break;
					case 0: /*** palette ***/
						export_palette ($f, $p, $id); break;
				}
			} else if (($id >= 41) and ($id <= 67)) {
				/*** 41..67: the 20 (regular) + 1 (Jaffar) + 1 (demo) + 5 (training) levels ***/
				export_level ($f, $p, $id);
			} else if (($id == 68) or ($id == 69)) {
				/*** 68: sprite palettes (27*16 colors) ***/
				/*** 69: screen palettes (33*16 colors) ***/
				export_palette ($f, $p, $id);
			} else if (($id == 70) or ($id == 73)) {
				/*** 70: hitpoint tiles (4 uncompressed 4bpp tiles) ***/
				/*** 73: princess room flame tiles (16 uncompressed 4bpp tiles) ***/
				export_tiles ($f, $p, $id);
			} else if (($id == 71) or ($id == 72) or
				(($id >= 75) and ($id <= 91))) {
				/*** 71: princess room screen ***/
				/*** 72: princess room arches screen ***/
				/*** 75: Masaya logo screen ... 91: Konami logo screen ***/
				export_screen ($f, $p, $id);
			} elseif ($id == 74) {
				/*** 74: ending font ***/
				export_blocks ($f, $p, $id, 2, 3, 8, 0x100, 0x200);
			}
			break;
		case 1: /*** Resource table 2. (sprites) ***/
			if (($id >= 0) and ($id <= 1248))
			{
				/*** regular sprites ***/
				export_sprite ($f, $p, $id);
			} else if ($id >= 1249 and $id <= 1484) {
				/* Sprites that are put into the background (traps, buttons,
				 * doors). 48*64 pixels
				 */
				export_bg_sprite ($f, $p, $id);
			}
			break;
		case 2: /*** Resource table 3. (sound and music) ***/
			break;
	}
}

/*****************************************************************************/

$EXIT_ERROR = 1;

//error_reporting (E_ALL);
error_reporting (~0); // all bits
ini_set ('display_errors', '1');
date_default_timezone_set ('UTC');
ini_set ('mbstring.internal_encoding', 'UTF-8');
mb_internal_encoding ('UTF-8');
set_time_limit (0);

$os = strtoupper (substr (PHP_OS, 0, 3));
switch ($os)
{
	case 'WIN': $GLOBALS['slash'] = '\\'; break;
	case 'LIN': $GLOBALS['slash'] = '/'; break;
	default:
		print ('Unknown OS: ' . $os . '!' . "\n");
		exit ($EXIT_ERROR);
}

if ($argc <= 1)
{
	print ('Usage: php pop1s_export.php <SMC file>' . "\n");
	exit ($EXIT_ERROR);
}

$fn = $argv[1];

if (!file_exists ($fn))
{
	print ('[FAILED] Could not find "' . $fn . '"!' . "\n");
	exit ($EXIT_ERROR);
}

$file_size = filesize ($fn);
if ($file_size != 0x100200 and $file_size != 0x100000)
{
	print ('ROM file "' . $fn . '" has a wrong size!' . "\n");
	exit ($EXIT_ERROR);
}

$f = file_get_contents($fn);

// Remove header if there is one.
if ($file_size === 0x100200){
	$f = substr ($f, 0x200);
}

/* Here we store the addresses of the three resource tables.
 * They are being stored in $ad[0][], $ad[1][] and $ad[2][].
 */
$ar = array (0x20000, 0x78000, 0xE8000);
$ad = array();
for ($ix = 0; $ix < count ($ar); $ix++)
{
	$ad[$ix] = array(); /*** We will store all addresses here. ***/
	$p = $ar[$ix];
	$mi = strlen ($f);
	$id = 0; /*** Start with address index 0. ***/
	while ($p < $mi)
	{
		$of = getint2 ($f, $p); /*** offset ***/
		$sg = getint1 ($f, $p); /*** segment (bank) ***/
		/*** Use the offset and segment to create the address. ***/
		$x = ($sg << 15) + ($of & 0x7FFF);
		$ad[$ix][$id] = $x; /*** Store the address. ***/
		if ($x < $mi) { $mi = $x; }
		$id++; /*** Next address index. ***/
	}
}

/* Here we loop through the addresses of each resource table and
 * call the functions export_font(), export_blocks(), export_palette(), export_tiles(),
 * export_screen(), export_sprite() and export_bg_sprite().
 */
for ($ix = 0; $ix < count ($ar); $ix++)
{
	$dir_to_make = "exp" . $ix;
	if (file_exists ($dir_to_make) == TRUE)
	{
		print ('Please delete the exp directories first!' . "\n");
		//exit ($EXIT_ERROR);
		continue;
	}
	mkdir ($dir_to_make);
	for ($id = 0; $id < count ($ad[$ix]); $id++)
	{
		export($ix, $id);
	}
}

?>